@@ -35,6 +35,21 @@
 #include "includes.h"
 #if !defined(HAVE_OPENPTY)
 
+#include <sys/types.h>
+
+#include <stdlib.h>
+
+#ifdef HAVE_SYS_STAT_H
+# include <sys/stat.h>
+#endif
+#ifdef HAVE_SYS_IOCTL_H
+# include <sys/ioctl.h>
+#endif
+
+#ifdef HAVE_FCNTL_H
+# include <fcntl.h>
+#endif
+
 #ifdef HAVE_UTIL_H
 # include <util.h>
 #endif /* HAVE_UTIL_H */
@@ -46,6 +61,10 @@
 # include <sys/stropts.h>
 #endif
 
+#include <signal.h>
+#include <string.h>
+#include <unistd.h>
+
 #ifndef O_NOCTTY
 #define O_NOCTTY 0
 #endif
@@ -103,7 +122,7 @@ openpty(int *amaster, int *aslave, char *name, struct termios *termp,
 	}
 
 	/*
-	 * Try to push the appropriate streams modules, as described 
+	 * Try to push the appropriate streams modules, as described
 	 * in Solaris pts(7).
 	 */
 	ioctl(*aslave, I_PUSH, "ptem");
@@ -165,7 +184,7 @@ openpty(int *amaster, int *aslave, char *name, struct termios *termp,
 	struct termios tio;
 
 	for (i = 0; i < num_ptys; i++) {
-		snprintf(ptbuf, sizeof(ptbuf), "/dev/pty%c%c", 
+		snprintf(ptbuf, sizeof(ptbuf), "/dev/pty%c%c",
 		    ptymajors[i / num_minors], ptyminors[i % num_minors]);
 		snprintf(ttbuf, sizeof(ttbuf), "/dev/tty%c%c",
 		    ptymajors[i / num_minors], ptyminors[i % num_minors]);
