@@ -67,7 +67,8 @@ static size_t argv_env_len = 0;
 void
 compat_init_setproctitle(int argc, char *argv[])
 {
-#if defined(SPT_TYPE) && SPT_TYPE == SPT_REUSEARGV
+#if !defined(HAVE_SETPROCTITLE) && \
+    defined(SPT_TYPE) && SPT_TYPE == SPT_REUSEARGV
 	extern char **environ;
 	char *lastargv = NULL;
 	char **envp = environ;
@@ -75,7 +76,7 @@ compat_init_setproctitle(int argc, char *argv[])
 
 	/*
 	 * NB: This assumes that argv has already been copied out of the
-	 * way. This is true for sshd, but may not be true for other 
+	 * way. This is true for sshd, but may not be true for other
 	 * programs. Beware.
 	 */
 
@@ -91,7 +92,7 @@ compat_init_setproctitle(int argc, char *argv[])
 	}
 
 	/*
-	 * Find the last argv string or environment variable within 
+	 * Find the last argv string or environment variable within
 	 * our process memory area.
 	 */
 	for (i = 0; i < argc; i++) {
@@ -107,8 +108,8 @@ compat_init_setproctitle(int argc, char *argv[])
 	argv_start = argv[0];
 	argv_env_len = lastargv - argv[0] - 1;
 
-	/* 
-	 * Copy environment 
+	/*
+	 * Copy environment
 	 * XXX - will truncate env on strdup fail
 	 */
 	for (i = 0; envp[i] != NULL; i++)
@@ -125,6 +126,7 @@ setproctitle(const char *fmt, ...)
 	va_list ap;
 	char buf[1024], ptitle[1024];
 	size_t len;
+	int r;
 	extern char *__progname;
 #if SPT_TYPE == SPT_PSTAT
 	union pstun pst;
@@ -137,13 +139,16 @@ setproctitle(const char *fmt, ...)
 
 	strlcpy(buf, __progname, sizeof(buf));
 
+	r = -1;
 	va_start(ap, fmt);
 	if (fmt != NULL) {
 		len = strlcat(buf, ": ", sizeof(buf));
 		if (len < sizeof(buf))
-			vsnprintf(buf + len, sizeof(buf) - len , fmt, ap);
+			r = vsnprintf(buf + len, sizeof(buf) - len , fmt, ap);
 	}
 	va_end(ap);
+	if (r == -1 || (size_t)r >= sizeof(buf) - len)
+		return;
 	strnvis(ptitle, buf, sizeof(ptitle),
 	    VIS_CSTYLE|VIS_NL|VIS_TAB|VIS_OCTAL);
 
@@ -151,7 +156,7 @@ setproctitle(const char *fmt, ...)
 	pst.pst_command = ptitle;
 	pstat(PSTAT_SETCMD, pst, strlen(ptitle), 0, 0);
 #elif SPT_TYPE == SPT_REUSEARGV
-/*	debug("setproctitle: copy \"%s\" into len %d", 
+/*	debug("setproctitle: copy \"%s\" into len %d",
 	    buf, argv_env_len); */
 	len = strlcpy(argv_start, ptitle, argv_env_len);
 	for(; len < argv_env_len; len++)
