@@ -1,4 +1,4 @@
-/*	$OpenBSD: sshbuf.h,v 1.7 2016/05/02 08:49:03 djm Exp $	*/
+/*	$OpenBSD: sshbuf.h,v 1.8 2016/11/25 23:22:04 djm Exp $	*/
 /*
  * Copyright (c) 2011 Damien Miller
  *
@@ -134,6 +134,14 @@ u_char *sshbuf_mutable_ptr(const struct sshbuf *buf);
  */
 int	sshbuf_check_reserve(const struct sshbuf *buf, size_t len);
 
+/*
+ * Preallocates len additional bytes in buf.
+ * Useful for cases where the caller knows how many bytes will ultimately be
+ * required to avoid realloc in the buffer code.
+ * Returns 0 on success, or a negative SSH_ERR_* error code on failure.
+ */
+int	sshbuf_allocate(struct sshbuf *buf, size_t len);
+
 /*
  * Reserve len bytes in buf.
  * Returns 0 on success and a pointer to the first reserved byte via the
